import { join } from 'path';
import { existsSync } from 'fs';
import { defineNuxtModule, createResolver, addPlugin, addComponent, addAutoImport, extendPages } from '@nuxt/kit';
import * as prismicVue from '@prismicio/vue';
import consola from 'consola';

const name = "@nuxtjs/prismic";
const version = "3.0.0-alpha.1";

const logger = consola.withScope("nuxt:prismic");

const module = defineNuxtModule({
  meta: {
    name: name,
    version: version,
    configKey: "prismic",
    compatibility: { nuxt: "^3.0.0" }
  },
  defaults: {
    endpoint: "",
    clientConfig: {},
    linkResolver: void 0,
    htmlSerializer: void 0,
    injectComponents: true,
    components: {
      linkInternalComponent: "nuxt-link"
    },
    preview: "/preview"
  },
  hooks: {},
  setup(mergedOptions, nuxt) {
    var _a, _b, _c;
    if (!mergedOptions.client && !mergedOptions.endpoint) {
      logger.warn("Options `endpoint` or `client` are required, disabling module...");
      return;
    }
    const resolver = createResolver(import.meta.url);
    nuxt.options.build.transpile.push(resolver.resolve("runtime"));
    (_a = nuxt.options).publicRuntimeConfig || (_a.publicRuntimeConfig = {});
    nuxt.options.publicRuntimeConfig[name] = mergedOptions;
    addPlugin(resolver.resolve("runtime/plugin"));
    if (mergedOptions.injectComponents) {
      [
        "PrismicEmbed",
        "PrismicImage",
        "PrismicLink",
        "PrismicText",
        "PrismicRichText",
        "SliceZone"
      ].forEach((component) => {
        addComponent({
          name: component,
          export: component,
          filePath: "@prismicio/vue"
        });
      });
    }
    const composableAutoImports = Object.keys(prismicVue).filter((key) => key.startsWith("use")).map((key) => {
      return {
        name: key,
        as: key,
        from: "@prismicio/vue"
      };
    });
    addAutoImport(composableAutoImports);
    if (mergedOptions.preview) {
      const userPreviewPagePath = join(nuxt.options.srcDir, nuxt.options.dir.pages, `${mergedOptions.preview}.vue`);
      if (existsSync(userPreviewPagePath)) {
        logger.info(`Using user-defined preview page at \`${userPreviewPagePath.replace(join(nuxt.options.rootDir), "~~").replace(/\\/g, "/")}\`, available at \`${mergedOptions.preview}\``);
      } else {
        logger.info(`Using default preview page, available at \`${mergedOptions.preview}\``);
        extendPages((pages) => {
          pages.unshift({
            name: "prismic-preview",
            path: mergedOptions.preview,
            file: resolver.resolve("runtime/preview.vue")
          });
        });
      }
      let repositoryName = "";
      if (mergedOptions.client) {
        repositoryName = new URL(mergedOptions.client.endpoint).host.split(".")[0];
      } else {
        try {
          repositoryName = new URL(mergedOptions.endpoint).host.split(".")[0];
        } catch (error) {
          repositoryName = mergedOptions.endpoint;
        }
      }
      (_b = nuxt.options).meta || (_b.meta = {});
      (_c = nuxt.options.meta).script || (_c.script = []);
      nuxt.options.meta.script.push({
        src: `https://static.cdn.prismic.io/prismic.min.js?repo=${repositoryName}&new=true`
      });
    }
  }
});

export { module as default };
